@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -707,11 +707,26 @@ class MAGNUM_GL_EXPORT AbstractFramebuffer {
     #else
     protected:
     #endif
-        explicit AbstractFramebuffer(): _flags{ObjectFlag::DeleteOnDestruction} {}
-        explicit AbstractFramebuffer(GLuint id, const Range2Di& viewport, ObjectFlags flags) noexcept: _id{id}, _viewport{viewport}, _flags{flags} {}
+        /* Used by the (constexpr) DefaultFramebuffer constructor and both
+           the NoCreate and normal constructor of Framebuffer */
+        constexpr explicit AbstractFramebuffer(GLuint id, const Range2Di& viewport, ObjectFlags flags) noexcept: _id{id}, _viewport{viewport}, _flags{flags} {}
 
         ~AbstractFramebuffer() = default;
 
+        AbstractFramebuffer(const AbstractFramebuffer&) = delete;
+        AbstractFramebuffer(AbstractFramebuffer&& other) noexcept: _id{other._id}, _viewport{other._viewport}, _flags{other._flags} {
+            other._id = 0;
+            other._viewport = {};
+        }
+        AbstractFramebuffer& operator=(const AbstractFramebuffer&) = delete;
+        AbstractFramebuffer& operator=(AbstractFramebuffer&& other) noexcept {
+            using std::swap;
+            swap(_id, other._id);
+            swap(_viewport, other._viewport);
+            swap(_flags, other._flags);
+            return *this;
+        }
+
         void MAGNUM_GL_LOCAL createIfNotAlready();
 
         void MAGNUM_GL_LOCAL bindInternal(FramebufferTarget target);
