@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -186,7 +186,7 @@ typedef CORRADE_DEPRECATED("use GL::TimeQuery instead") Magnum::GL::TimeQuery Ti
 
 }
 #else
-#error this header is not available in WebGL 1.0 build
+#error this header is not available in WebGL build
 #endif
 
 #endif
