@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -34,31 +34,16 @@
 #include "Magnum/Types.h"
 #include "Magnum/Math/Math.h"
 
-#if defined(MAGNUM_BUILD_DEPRECATED) && defined(MAGNUM_TARGET_GL)
+#ifdef MAGNUM_BUILD_DEPRECATED
+#include <Corrade/Utility/Macros.h>
+
+#ifdef MAGNUM_TARGET_GL
 #include "Magnum/GL/GL.h"
 #endif
-
-#ifndef DOXYGEN_GENERATING_OUTPUT
-typedef unsigned int GLenum; /* Needed for *Format and *Type enums */
 #endif
 
 namespace Magnum {
 
-namespace Math {
-    #ifndef DOXYGEN_GENERATING_OUTPUT
-    template<class> struct Constants;
-
-    #ifdef MAGNUM_BUILD_DEPRECATED
-    namespace Literals {
-        constexpr Rad<Double> operator "" _rad(long double);
-        constexpr Deg<Double> operator "" _deg(long double);
-        constexpr Rad<Float> operator "" _radf(long double);
-        constexpr Deg<Float> operator "" _degf(long double);
-    }
-    #endif
-    #endif
-}
-
 /* Bring whole Corrade namespace */
 using namespace Corrade;
 
@@ -706,13 +691,6 @@ typedef Math::Frustum<Double> Frustumd;
 
 /*@}*/
 
-#ifdef MAGNUM_BUILD_DEPRECATED
-using Math::Literals::operator "" _deg;
-using Math::Literals::operator "" _rad;
-using Math::Literals::operator "" _degf;
-using Math::Literals::operator "" _radf;
-#endif
-
 #ifndef DOXYGEN_GENERATING_OUTPUT
 template<UnsignedInt, class T> class Array;
 template<class T> class Array1D;
