@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -33,6 +33,7 @@
 #include <Corrade/Utility/Debug.h>
 
 #include "Magnum/Types.h"
+#include "Magnum/Math/Math.h"
 #include "Magnum/Math/Tags.h"
 
 namespace Magnum { namespace Math {
@@ -303,6 +304,28 @@ template<std::size_t size> inline BoolVector<size> BoolVector<size>::operator~()
     return out;
 }
 
+namespace Implementation {
+
+template<std::size_t size> struct StrictWeakOrdering<BoolVector<size>> {
+    bool operator()(const BoolVector<size>& a, const BoolVector<size>& b) const {
+        auto ad = a.data();
+        auto bd = b.data();
+        for(std::size_t i = 0; i < BoolVector<size>::DataSize - 1; ++i) {
+            if(ad[i] < bd[i])
+                return true;
+            if(ad[i] > bd[i])
+                return false;
+        }
+
+        /* Mask last element with to hide padding bits */
+        constexpr UnsignedByte mask = UnsignedByte(0xFF) >> (BoolVector<size>::DataSize * 8 - size);
+        constexpr std::size_t i = BoolVector<size>::DataSize - 1;
+        return (ad[i] & mask) < (bd[i] & mask);
+    }
+};
+
+}
+
 }}
 
 #endif
