@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -53,8 +53,20 @@ namespace Implementation {
         template<class T> constexpr static T pow(T) { return T(1); }
     };
 
-    template<class> struct IsBoolVector: std::false_type {};
-    template<std::size_t size> struct IsBoolVector<BoolVector<size>>: std::true_type {};
+    template<class> struct IsBoolVectorOrScalar: std::false_type {};
+    template<> struct IsBoolVectorOrScalar<bool>: std::true_type {};
+    template<std::size_t size> struct IsBoolVectorOrScalar<BoolVector<size>>: std::true_type {};
+
+    template<class T> struct IsVectorOrScalar: std::is_arithmetic<T>::type {};
+    template<template<class> class Derived, class T> struct IsVectorOrScalar<Unit<Derived, T>>: std::true_type {};
+    template<class T> struct IsVectorOrScalar<Deg<T>>: std::true_type {};
+    template<class T> struct IsVectorOrScalar<Rad<T>>: std::true_type {};
+    template<std::size_t size, class T> struct IsVectorOrScalar<Vector<size, T>>: std::true_type {};
+    template<class T> struct IsVectorOrScalar<Vector2<T>>: std::true_type {};
+    template<class T> struct IsVectorOrScalar<Vector3<T>>: std::true_type {};
+    template<class T> struct IsVectorOrScalar<Vector4<T>>: std::true_type {};
+    template<class T> struct IsVectorOrScalar<Color3<T>>: std::true_type {};
+    template<class T> struct IsVectorOrScalar<Color4<T>>: std::true_type {};
 }
 
 /**
@@ -79,7 +91,7 @@ UnsignedInt MAGNUM_EXPORT log2(UnsignedInt number);
 Returns natural (base @f$ e @f$) logarithm of given number.
 @see @ref Constants::e(), @ref log(UnsignedInt, UnsignedInt), @ref log2()
 */
-template<class T> T log(T number) { return std::log(number); }
+template<class T> inline T log(T number) { return std::log(number); }
 
 /**
 @brief Natural exponential
@@ -87,26 +99,20 @@ template<class T> T log(T number) { return std::log(number); }
 Returns @f$ e^x @f$.
 @see @ref Constants::e(), @ref pow(T, T)
 */
-template<class T> T exp(T exponent) { return std::exp(exponent); }
+template<class T> inline T exp(T exponent) { return std::exp(exponent); }
 
 /**
 @brief Integer division with remainder
 
 Example usage:
 
-@code{.cpp}
-Int quotient, remainder;
-std::tie(quotient, remainder) = Math::div(57, 6); // {9, 3}
-@endcode
+@snippet MagnumMath.cpp div
 
 Equivalent to the following, but possibly done in a single CPU instruction:
 
-@code{.cpp}
-Int quotient = 57/6;
-Int remainder = 57%6;
-@endcode
+@snippet MagnumMath.cpp div-equivalent
 */
-template<class Integral> std::pair<Integral, Integral> div(Integral x, Integral y) {
+template<class Integral> inline std::pair<Integral, Integral> div(Integral x, Integral y) {
     static_assert(std::is_integral<Integral>{}, "Math::div(): not an integral type");
     const auto result = std::div(x, y);
     return {result.quot, result.rem};
@@ -117,15 +123,35 @@ template<class Integral> std::pair<Integral, Integral> div(Integral x, Integral
 
 @see @ref isNan(), @ref Constants::inf()
 */
-template<class T> bool isInf(T value) { return std::isinf(value); }
+template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, bool>::type isInf(T value) {
+    return std::isinf(value);
+}
+
+/** @overload */
+template<std::size_t size, class T> inline BoolVector<size> isInf(const Vector<size, T>& value) {
+    BoolVector<size> out;
+    for(std::size_t i = 0; i != size; ++i)
+        out.set(i, std::isinf(value[i]));
+    return out;
+}
 
 /**
-@brief If given number is NaN
+@brief If given number is a NaN
 
 Equivalent to @cpp value != value @ce.
 @see @ref isInf(), @ref Constants::nan()
 */
-template<class T> bool isNan(T value) { return std::isnan(value); }
+template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, bool>::type isNan(T value) {
+    return std::isnan(value);
+}
+
+/** @overload */
+template<std::size_t size, class T> inline BoolVector<size> isNan(const Vector<size, T>& value) {
+    BoolVector<size> out;
+    for(std::size_t i = 0; i != size; ++i)
+        out.set(i, std::isnan(value[i]));
+    return out;
+}
 
 /** @todo Can't trigonometric functions be done with only one overload? */
 
@@ -208,7 +234,7 @@ template<UnsignedInt exponent, class T> constexpr T pow(T base);
 template<UnsignedInt exponent, class T> constexpr typename std::enable_if<std::is_arithmetic<T>::value, T>::type pow(T base) {
     return Implementation::Pow<exponent>::pow(base);
 }
-template<UnsignedInt exponent, std::size_t size, class T> Vector<size, T> pow(const Vector<size, T>& base) {
+template<UnsignedInt exponent, std::size_t size, class T> inline Vector<size, T> pow(const Vector<size, T>& base) {
     Vector<size, T> out{NoInit};
     for(std::size_t i = 0; i != size; ++i)
         out[i] = Implementation::Pow<exponent>::pow(base[i]);
@@ -225,7 +251,7 @@ Returns power of @p base to the @p exponent.
 #ifdef DOXYGEN_GENERATING_OUTPUT
 template<class T> T pow(T base, T exponent);
 #else
-template<class T> typename std::enable_if<std::is_arithmetic<T>::value, T>::type pow(T base, T exponent) {
+template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, T>::type pow(T base, T exponent) {
     return std::pow(base, exponent);
 }
 template<std::size_t size, class T> inline Vector<size, T> pow(const Vector<size, T>& base, T exponent) {
@@ -269,7 +295,7 @@ template<std::size_t size, class T> inline Vector<size, T> min(const Vector<size
 
 If the range is empty, returns default-constructed value.
 */
-template<class T> T min(Corrade::Containers::ArrayView<const T> range) {
+template<class T> inline T min(Corrade::Containers::ArrayView<const T> range) {
     if(range.empty()) return {};
 
     T out(range[0]);
@@ -283,6 +309,11 @@ template<class T> inline T min(std::initializer_list<T> list) {
     return min(Corrade::Containers::ArrayView<const T>{list.begin(), list.size()});
 }
 
+/** @overload */
+template<class T, std::size_t size> inline T min(const T(&array)[size]) {
+    return min(Corrade::Containers::arrayView(array));
+}
+
 /**
 @brief Maximum
 
@@ -304,7 +335,7 @@ template<std::size_t size, class T> Vector<size, T> max(const Vector<size, T>& v
 #endif
 
 /** @overload */
-template<std::size_t size, class T> Vector<size, T> max(const Vector<size, T>& value, T max) {
+template<std::size_t size, class T> inline Vector<size, T> max(const Vector<size, T>& value, T max) {
     Vector<size, T> out{NoInit};
     for(std::size_t i = 0; i != size; ++i)
         out[i] = std::max(value[i], max);
@@ -316,7 +347,7 @@ template<std::size_t size, class T> Vector<size, T> max(const Vector<size, T>& v
 
 If the range is empty, returns default-constructed value.
 */
-template<class T> T max(Corrade::Containers::ArrayView<const T> range) {
+template<class T> inline T max(Corrade::Containers::ArrayView<const T> range) {
     if(range.empty()) return {};
 
     T out(range[0]);
@@ -330,10 +361,16 @@ template<class T> inline T max(std::initializer_list<T> list) {
     return max(Corrade::Containers::ArrayView<const T>{list.begin(), list.size()});
 }
 
+/** @overload */
+template<class T, std::size_t size> inline T max(const T(&array)[size]) {
+    return max(Corrade::Containers::arrayView(array));
+}
+
 /**
 @brief Minimum and maximum of two values
 
-@see @ref min(), @ref max(), @ref clamp(), @ref Vector::minmax()
+@see @ref min(), @ref max(), @ref clamp(), @ref Vector::minmax(),
+    @ref Range::Range(const std::pair<VectorType, VectorType>&)
 */
 #ifdef DOXYGEN_GENERATING_OUTPUT
 template<class T> inline std::pair<T, T> minmax(const T& a, const T& b);
@@ -341,7 +378,7 @@ template<class T> inline std::pair<T, T> minmax(const T& a, const T& b);
 template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, std::pair<T, T>>::type minmax(T a, T b) {
     return a < b ? std::make_pair(a, b) : std::make_pair(b, a);
 }
-template<std::size_t size, class T> std::pair<Vector<size, T>, Vector<size, T>> minmax(const Vector<size, T>& a, const Vector<size, T>& b) {
+template<std::size_t size, class T> inline std::pair<Vector<size, T>, Vector<size, T>> minmax(const Vector<size, T>& a, const Vector<size, T>& b) {
     using std::swap;
     std::pair<Vector<size, T>, Vector<size, T>> out{a, b};
     for(std::size_t i = 0; i != size; ++i)
@@ -367,8 +404,9 @@ namespace Implementation {
 @brief Minimum and maximum of a range
 
 If the range is empty, returns default-constructed values.
+@see @ref Range::Range(const std::pair<VectorType, VectorType>&)
 */
-template<class T> std::pair<T, T> minmax(Corrade::Containers::ArrayView<const T> range) {
+template<class T> inline std::pair<T, T> minmax(Corrade::Containers::ArrayView<const T> range) {
     if(range.empty()) return {};
 
     T min{range[0]}, max{range[0]};
@@ -383,15 +421,18 @@ template<class T> inline std::pair<T, T> minmax(std::initializer_list<T> list) {
     return minmax(Corrade::Containers::ArrayView<const T>{list.begin(), list.size()});
 }
 
+/** @overload */
+template<class T, std::size_t size> inline std::pair<T, T> minmax(const T(&array)[size]) {
+    return minmax(Corrade::Containers::arrayView(array));
+}
+
 /**
 @brief Clamp value
 
 Values smaller than @p min are set to @p min, values larger than @p max are
 set to @p max. Equivalent to:
 
-@code{.cpp}
-Math::min(Math::max(value, min), max)
-@endcode
+@snippet MagnumMath.cpp clamp
 
 <em>NaN</em>s passed in @p value parameter are propagated.
 @see @ref min(), @ref max()
@@ -402,7 +443,7 @@ template<class T, class U> inline T clamp(const T& value, const T& min, const T&
 template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, T>::type clamp(T value, T min, T max) {
     return std::min(std::max(value, min), max);
 }
-template<std::size_t size, class T> Vector<size, T> clamp(const Vector<size, T>& value, const Vector<size, T>& min, const Vector<size, T>& max) {
+template<std::size_t size, class T> inline Vector<size, T> clamp(const Vector<size, T>& value, const Vector<size, T>& min, const Vector<size, T>& max) {
     Vector<size, T> out{NoInit};
     for(std::size_t i = 0; i != size; ++i)
         out[i] = clamp(value[i], min[i], max[i]);
@@ -411,7 +452,7 @@ template<std::size_t size, class T> Vector<size, T> clamp(const Vector<size, T>&
 #endif
 
 /** @overload */
-template<std::size_t size, class T> Vector<size, T> clamp(const Vector<size, T>& value, T min, T max) {
+template<std::size_t size, class T> inline Vector<size, T> clamp(const Vector<size, T>& value, T min, T max) {
     Vector<size, T> out{NoInit};
     for(std::size_t i = 0; i != size; ++i)
         out[i] = clamp(value[i], min, max);
@@ -431,7 +472,7 @@ template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, T
     if(scalar < T(0)) return T(-1);
     return T(0);
 }
-template<std::size_t size, class T> Vector<size, T> sign(const Vector<size, T>& a) {
+template<std::size_t size, class T> inline Vector<size, T> sign(const Vector<size, T>& a) {
     Vector<size, T> out{NoInit};
     for(std::size_t i = 0; i != size; ++i)
         out[i] = sign(a[i]);
@@ -446,7 +487,7 @@ template<class T> inline T abs(const T& a);
 template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, T>::type abs(T a) {
     return std::abs(a);
 }
-template<std::size_t size, class T> Vector<size, T> abs(const Vector<size, T>& a) {
+template<std::size_t size, class T> inline Vector<size, T> abs(const Vector<size, T>& a) {
     Vector<size, T> out{NoInit};
     for(std::size_t i = 0; i != size; ++i)
         out[i] = std::abs(a[i]);
@@ -461,7 +502,7 @@ template<class T> inline T floor(const T& a);
 template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, T>::type floor(T a) {
     return std::floor(a);
 }
-template<std::size_t size, class T> Vector<size, T> floor(const Vector<size, T>& a) {
+template<std::size_t size, class T> inline Vector<size, T> floor(const Vector<size, T>& a) {
     Vector<size, T> out{NoInit};
     for(std::size_t i = 0; i != size; ++i)
         out[i] = std::floor(a[i]);
@@ -476,7 +517,7 @@ template<class T> inline T round(const T& a);
 template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, T>::type round(T a) {
     return std::round(a);
 }
-template<std::size_t size, class T> Vector<size, T> round(const Vector<size, T>& a) {
+template<std::size_t size, class T> inline Vector<size, T> round(const Vector<size, T>& a) {
     Vector<size, T> out{NoInit};
     for(std::size_t i = 0; i != size; ++i)
         out[i] = std::round(a[i]);
@@ -491,7 +532,7 @@ template<class T> inline T ceil(const T& a);
 template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, T>::type ceil(T a) {
     return std::ceil(a);
 }
-template<std::size_t size, class T> Vector<size, T> ceil(const Vector<size, T>& a) {
+template<std::size_t size, class T> inline Vector<size, T> ceil(const Vector<size, T>& a) {
     Vector<size, T> out{NoInit};
     for(std::size_t i = 0; i != size; ++i)
         out[i] = std::ceil(a[i]);
@@ -510,7 +551,7 @@ template<class T> inline T sqrt(const T& a);
 template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, T>::type sqrt(T a) {
     return T(std::sqrt(a));
 }
-template<std::size_t size, class T> Vector<size, T> sqrt(const Vector<size, T>& a) {
+template<std::size_t size, class T> inline Vector<size, T> sqrt(const Vector<size, T>& a) {
     Vector<size, T> out{NoInit};
     for(std::size_t i = 0; i != size; ++i)
         out[i] = T(std::sqrt(a[i]));
@@ -530,7 +571,7 @@ template<class T> inline T sqrtInverted(const T& a);
 template<class T> inline typename std::enable_if<std::is_arithmetic<T>::value, T>::type sqrtInverted(T a) {
     return T(1)/std::sqrt(a);
 }
-template<std::size_t size, class T> Vector<size, T> sqrtInverted(const Vector<size, T>& a) {
+template<std::size_t size, class T> inline Vector<size, T> sqrtInverted(const Vector<size, T>& a) {
     return Vector<size, T>(T(1))/sqrt(a);
 }
 #endif
@@ -542,21 +583,34 @@ template<std::size_t size, class T> Vector<size, T> sqrtInverted(const Vector<si
 @param t     Interpolation phase (from range @f$ [0; 1] @f$)
 
 The interpolation for vectors is done as in following, similarly for scalars: @f[
-    \boldsymbol v_{LERP} = (1 - t) \boldsymbol v_A + t \boldsymbol v_B
+    \boldsymbol{v_{LERP}} = (1 - t) \boldsymbol{v_A} + t \boldsymbol{v_B}
 @f]
-@see @ref lerpInverted(), @ref lerp(const Quaternion<T>&, const Quaternion<T>&, T)
+
+See @ref select() for constant interpolation using the same API and
+@ref splerp() for spline interpolation.
+@see @ref lerpInverted(), @ref lerp(const Complex<T>&, const Complex<T>&, T),
+    @ref lerp(const Quaternion<T>&, const Quaternion<T>&, T),
+    @ref lerp(const CubicHermite<T>&, const CubicHermite<T>&, U),
+    @ref lerp(const CubicHermiteComplex<T>&, const CubicHermiteComplex<T>&, T),
+    @ref lerp(const CubicHermiteQuaternion<T>&, const CubicHermiteQuaternion<T>&, T)
 @m_keyword{mix(),GLSL mix(),}
 */
-#ifdef DOXYGEN_GENERATING_OUTPUT
-template<class T, class U> inline T lerp(const T& a, const T& b, U t);
-#else
-template<class T, class U> inline typename std::enable_if<!Implementation::IsBoolVector<U>::value, T>::type lerp(T a, T b, U t) {
-    return T(Implementation::lerp(a, b, t));
-}
-template<std::size_t size, class T, class U> inline typename std::enable_if<!Implementation::IsBoolVector<U>::value, Vector<size, T>>::type lerp(const Vector<size, T>& a, const Vector<size, T>& b, U t) {
+template<class T, class U> inline
+    #ifndef DOXYGEN_GENERATING_OUTPUT
+    typename std::enable_if<Implementation::IsVectorOrScalar<T>::value && !Implementation::IsBoolVectorOrScalar<U>::value, T>::type
+    #else
+    T
+    #endif
+lerp(const T& a, const T& b, U t) {
     return Implementation::lerp(a, b, t);
 }
-#endif
+
+/** @overload
+@m_keyword{mix(),GLSL mix(),}
+*/
+template<class T> inline T lerp(const T& a, const T& b, bool t) {
+    return t ? b : a;
+}
 
 /** @overload
 Similar to the above, but instead of multiplication and addition it just does
@@ -589,9 +643,9 @@ template<std::size_t size> inline BoolVector<size> lerp(const BoolVector<size>&
 @param lerp Interpolated value
 
 Returns interpolation phase *t*: @f[
-    t = \frac{\boldsymbol v_{LERP} - \boldsymbol v_A}{\boldsymbol v_B - \boldsymbol v_A}
+    t = \frac{\boldsymbol{v_{LERP}} - \boldsymbol{v_A}}{\boldsymbol{v_B} - \boldsymbol{v_A}}
 @f]
-@see @ref lerp()
+@see @ref lerp(), @ref select()
 */
 #ifdef DOXYGEN_GENERATING_OUTPUT
 template<class T> inline T lerpInverted(const T& a, const T& b, const T& lerp);
@@ -604,6 +658,25 @@ template<std::size_t size, class T, class U> inline Vector<size, T> lerpInverted
 }
 #endif
 
+/**
+@brief Constant interpolation of two values
+@param a     First value
+@param b     Second value
+@param t     Interpolation phase
+
+A constant interpolation counterpart to @ref lerp(): @f[
+    \boldsymbol{v}_i = \begin{cases}
+        \boldsymbol{v_A}_i, & t_i < 1 \\
+        \boldsymbol{v_B}_i, & t_i \ge 1
+    \end{cases}
+@f]
+
+Equivalent to calling @cpp Math::lerp(a, b, t >= U(1)) @ce.
+*/
+template<class T, class U> constexpr T select(const T& a, const T& b, U t) {
+    return lerp(a, b, t >= U(1));
+}
+
 /**
 @brief Fused multiply-add
 
@@ -631,11 +704,4 @@ template<std::size_t size, class T> inline Vector<size, T> fma(const Vector<size
 
 }}
 
-#ifdef MAGNUM_BUILD_DEPRECATED
-/* In order to make the deprecated normalize() / denormalize() functions
-   available in the original header. The Packing.h header depends on this file
-   so it needs to be included after it. */
-#include "Magnum/Math/Packing.h"
-#endif
-
 #endif
