@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -26,7 +26,7 @@
 */
 
 /** @file
- * @brief Class @ref Magnum::Math::Quaternion
+ * @brief Class @ref Magnum::Math::Quaternion, function @ref Magnum::Math::dot(), @ref Magnum::Math::angle(), @ref Magnum::Math::lerp(), @ref Magnum::Math::slerp()
  */
 
 #include <cmath>
@@ -66,15 +66,15 @@ namespace Implementation {
 @brief Angle between normalized quaternions
 
 Expects that both quaternions are normalized. @f[
-     \theta = acos \left( \frac{p \cdot q}{|p| |q|} \right) = acos(p \cdot q)
+     \theta = \arccos \left( \frac{p \cdot q}{|p| |q|} \right) = \arccos(p \cdot q)
 @f]
 @see @ref Quaternion::isNormalized(),
     @ref angle(const Complex<T>&, const Complex<T>&),
-    @ref angle(const Vector<size, T>&, const Vector<size, T>&)
+    @ref angle(const Vector<size, FloatingPoint>&, const Vector<size, FloatingPoint>&)
  */
 template<class T> inline Rad<T> angle(const Quaternion<T>& normalizedA, const Quaternion<T>& normalizedB) {
     CORRADE_ASSERT(normalizedA.isNormalized() && normalizedB.isNormalized(),
-        "Math::angle(): quaternions must be normalized", {});
+        "Math::angle(): quaternions" << normalizedA << "and" << normalizedB << "are not normalized", {});
     return Rad<T>{Implementation::angle(normalizedA, normalizedB)};
 }
 
@@ -87,15 +87,51 @@ template<class T> inline Rad<T> angle(const Quaternion<T>& normalizedA, const Qu
 Expects that both quaternions are normalized. @f[
     q_{LERP} = \frac{(1 - t) q_A + t q_B}{|(1 - t) q_A + t q_B|}
 @f]
-@see @ref Quaternion::isNormalized(), @ref slerp(const Quaternion<T>&, const Quaternion<T>&, T),
-    @ref lerp(const T&, const T&, U)
+
+Note that this function does not check for shortest path interpolation, see
+@ref lerpShortestPath(const Quaternion<T>&, const Quaternion<T>&, T) for an
+alternative.
+@see @ref Quaternion::isNormalized(),
+    @ref slerp(const Quaternion<T>&, const Quaternion<T>&, T), @ref sclerp(),
+    @ref lerp(const T&, const T&, U),
+    @ref lerp(const Complex<T>&, const Complex<T>&, T),
+    @ref lerp(const CubicHermite<T>&, const CubicHermite<T>&, U),
+    @ref lerp(const CubicHermiteComplex<T>&, const CubicHermiteComplex<T>&, T),
+    @ref lerp(const CubicHermiteQuaternion<T>&, const CubicHermiteQuaternion<T>&, T)
 */
 template<class T> inline Quaternion<T> lerp(const Quaternion<T>& normalizedA, const Quaternion<T>& normalizedB, T t) {
     CORRADE_ASSERT(normalizedA.isNormalized() && normalizedB.isNormalized(),
-        "Math::lerp(): quaternions must be normalized", {});
+        "Math::lerp(): quaternions" << normalizedA << "and" << normalizedB << "are not normalized", {});
     return ((T(1) - t)*normalizedA + t*normalizedB).normalized();
 }
 
+/** @relatesalso Quaternion
+@brief Linear shortest-path interpolation of two quaternions
+@param normalizedA  First quaternion
+@param normalizedB  Second quaternion
+@param t            Interpolation phase (from range @f$ [0; 1] @f$)
+
+Unlike @ref lerp(const Quaternion<T>&, const Quaternion<T>&, T), this
+interpolates on the shortest path at some performance expense. Expects that
+both quaternions are normalized. @f[
+    \begin{array}{rcl}
+        d & = & q_A \cdot q_B \\[5pt]
+        q'_A & = & \begin{cases}
+                \phantom{-}q_A, & d \ge 0 \\
+                -q_A, & d < 0
+            \end{cases} \\[15pt]
+        q_{LERP} & = & \cfrac{(1 - t) q'_A + t q_B}{|(1 - t) q'_A + t q_B|}
+    \end{array}
+@f]
+@see @ref Quaternion::isNormalized(),
+    @ref slerpShortestPath(const Quaternion<T>&, const Quaternion<T>&, T),
+    @ref lerpShortestPath(const CubicHermiteQuaternion<T>&, const CubicHermiteQuaternion<T>&, T)
+    @ref sclerpShortestPath()
+*/
+template<class T> inline Quaternion<T> lerpShortestPath(const Quaternion<T>& normalizedA, const Quaternion<T>& normalizedB, T t) {
+    return lerp(dot(normalizedA, normalizedB) < T(0) ? -normalizedA : normalizedA, normalizedB, t);
+}
+
 /** @relatesalso Quaternion
 @brief Spherical linear interpolation of two quaternions
 @param normalizedA  First quaternion
@@ -103,84 +139,115 @@ template<class T> inline Quaternion<T> lerp(const Quaternion<T>& normalizedA, co
 @param t            Interpolation phase (from range @f$ [0; 1] @f$)
 
 Expects that both quaternions are normalized. If the quaternions are the same
-or one is a negation of the other, returns the first argument. @f[
-    q_{SLERP} = \frac{sin((1 - t) \theta) q_A + sin(t \theta) q_B}{sin \theta}
-    ~ ~ ~ ~ ~ ~ ~
-    \theta = acos \left( \frac{q_A \cdot q_B}{|q_A| \cdot |q_B|} \right) = acos(q_A \cdot q_B)
+or one is a negation of the other, it just returns the first argument: @f[
+    \begin{array}{rcl}
+        d & = & q_A \cdot q_B \\[5pt]
+        q_{SLERP} & = & q_A, ~ {\color{m-primary} \text{if} ~ d \ge 1}
+    \end{array}
 @f]
-@see @ref Quaternion::isNormalized(), @ref lerp(const Quaternion<T>&, const Quaternion<T>&, T)
- */
+
+@m_class{m-noindent}
+
+otherwise, the interpolation is performed as: @f[
+    \begin{array}{rcl}
+        \theta & = & \arccos \left( \frac{q_A \cdot q_B}{|q_A| |q_B|} \right) = \arccos(q_A \cdot q_B) = \arccos(d) \\[5pt]
+        q_{SLERP} & = & \cfrac{\sin((1 - t) \theta) q_A + \sin(t \theta) q_B}{\sin(\theta)}
+    \end{array}
+@f]
+
+Note that this function does not check for shortest path interpolation, see
+@ref slerpShortestPath(const Quaternion<T>&, const Quaternion<T>&, T) for an
+alternative.
+@see @ref Quaternion::isNormalized(), @ref lerp(const Quaternion<T>&, const Quaternion<T>&, T),
+    @ref slerp(const Complex<T>&, const Complex<T>&, T), @ref sclerp(),
+    @ref slerp(const CubicHermiteQuaternion<T>&, const CubicHermiteQuaternion<T>&, T)
+*/
 template<class T> inline Quaternion<T> slerp(const Quaternion<T>& normalizedA, const Quaternion<T>& normalizedB, T t) {
     CORRADE_ASSERT(normalizedA.isNormalized() && normalizedB.isNormalized(),
-        "Math::slerp(): quaternions must be normalized", {});
+        "Math::slerp(): quaternions" << normalizedA << "and" << normalizedB << "are not normalized", {});
     const T cosHalfAngle = dot(normalizedA, normalizedB);
 
     /* Avoid division by zero */
-    if(std::abs(cosHalfAngle) >= T(1)) return Quaternion<T>{normalizedA};
+    if(std::abs(cosHalfAngle) >= T(1) - TypeTraits<T>::epsilon())
+        return normalizedA;
 
     const T a = std::acos(cosHalfAngle);
     return (std::sin((T(1) - t)*a)*normalizedA + std::sin(t*a)*normalizedB)/std::sin(a);
 }
 
+/** @relatesalso Quaternion
+@brief Spherical linear shortest-path interpolation of two quaternions
+@param normalizedA  First quaternion
+@param normalizedB  Second quaternion
+@param t            Interpolation phase (from range @f$ [0; 1] @f$)
+
+Unlike @ref slerp(const Quaternion<T>&, const Quaternion<T>&, T) this function
+interpolates on the shortest path. Expects that both quaternions are
+normalized. If the quaternions are the same or one is a negation of the other,
+it just returns the first argument: @f[
+    \begin{array}{rcl}
+        d & = & q_A \cdot q_B \\
+        q_{SLERP} & = & q_A, ~ {\color{m-primary} \text{if} ~ d \ge 1}
+    \end{array}
+@f]
+
+@m_class{m-noindent}
+
+otherwise, the interpolation is performed as: @f[
+    \begin{array}{rcl}
+        q'_A & = & \begin{cases}
+                \phantom{-}q_A, & d \ge 0 \\
+                -q_A, & d < 0
+            \end{cases} \\[15pt]
+        \theta & = & \arccos \left( \frac{|q'_A \cdot q_B|}{|q'_A| |q_B|} \right) = \arccos(|q'_A \cdot q_B|) = \arccos(|d|) \\[5pt]
+        q_{SLERP} & = & \cfrac{\sin((1 - t) \theta) q'_A + \sin(t \theta) q_B}{\sin(\theta)}
+    \end{array}
+@f]
+@see @ref Quaternion::isNormalized(),
+    @ref lerpShortestPath(const Quaternion<T>&, const Quaternion<T>&, T),
+    @ref slerpShortestPath(const CubicHermiteQuaternion<T>&, const CubicHermiteQuaternion<T>&, T),
+    @ref sclerpShortestPath()
+*/
+template<class T> inline Quaternion<T> slerpShortestPath(const Quaternion<T>& normalizedA, const Quaternion<T>& normalizedB, T t) {
+    CORRADE_ASSERT(normalizedA.isNormalized() && normalizedB.isNormalized(),
+        "Math::slerpShortestPath(): quaternions" << normalizedA << "and" << normalizedB << "are not normalized", {});
+    const T cosHalfAngle = dot(normalizedA, normalizedB);
+
+    /* Avoid division by zero */
+    if(std::abs(cosHalfAngle) >= T(1) - TypeTraits<T>::epsilon())
+        return normalizedA;
+
+    const Quaternion<T> shortestNormalizedA = cosHalfAngle < 0 ? -normalizedA : normalizedA;
+
+    const T a = std::acos(std::abs(cosHalfAngle));
+    return (std::sin((T(1) - t)*a)*shortestNormalizedA + std::sin(t*a)*normalizedB)/std::sin(a);
+}
+
 /**
 @brief Quaternion
 @tparam T   Underlying data type
 
-Represents 3D rotation. See @ref transformations for brief introduction.
+Represents 3D rotation. Usually denoted as the following in equations, with
+@f$ \boldsymbol{q}_V @f$ being the @ref vector() part and @f$ q_S @f$ being the
+@ref scalar() part: @f[
+    q = [\boldsymbol{q}_V, q_S]
+@f]
+
+See @ref transformations for a brief introduction.
 @see @ref Magnum::Quaternion, @ref Magnum::Quaterniond, @ref DualQuaternion,
     @ref Matrix4
 */
 template<class T> class Quaternion {
-    template<class> friend class Quaternion;
-
     public:
         typedef T Type; /**< @brief Underlying data type */
 
-        #ifdef MAGNUM_BUILD_DEPRECATED
-        /**
-         * @brief @copybrief Math::dot(const Quaternion<T>&, const Quaternion<T>&)
-         * @deprecated Use @ref Math::dot(const Quaternion<T>&, const Quaternion<T>&)
-         *      instead.
-         */
-        CORRADE_DEPRECATED("use Math::dot() instead") static T dot(const Quaternion<T>& a, const Quaternion<T>& b) {
-            return Math::dot(a, b);
-        }
-
-        /**
-         * @brief @copybrief Math::angle(const Quaternion<T>&, const Quaternion<T>&)
-         * @deprecated Use @ref Math::angle(const Quaternion<T>&, const Quaternion<T>&)
-         *      instead.
-         */
-        CORRADE_DEPRECATED("use Math::angle() instead") static Rad<T> angle(const Quaternion<T>& normalizedA, const Quaternion<T>& normalizedB) {
-            return Math::angle(normalizedA, normalizedB);
-        }
-
-        /**
-         * @brief @copybrief Math::lerp(const Quaternion<T>&, const Quaternion<T>&, T)
-         * @deprecated Use @ref Math::lerp(const Quaternion<T>&, const Quaternion<T>&, T)
-         *      instead.
-         */
-        CORRADE_DEPRECATED("use Math::lerp() instead") static Quaternion<T> lerp(const Quaternion<T>& normalizedA, const Quaternion<T>& normalizedB, T t) {
-            return Math::lerp(normalizedA, normalizedB, t);
-        }
-
-        /**
-         * @brief @copybrief Math::slerp(const Quaternion<T>&, const Quaternion<T>&, T)
-         * @deprecated Use @ref Math::slerp(const Quaternion<T>&, const Quaternion<T>&, T)
-         *      instead.
-         */
-        CORRADE_DEPRECATED("use Math::slerp() instead") static Quaternion<T> slerp(const Quaternion<T>& normalizedA, const Quaternion<T>& normalizedB, T t) {
-            return Math::slerp(normalizedA, normalizedB, t);
-        }
-        #endif
-
         /**
          * @brief Rotation quaternion
          * @param angle             Rotation angle (counterclockwise)
          * @param normalizedAxis    Normalized rotation axis
          *
          * Expects that the rotation axis is normalized. @f[
-         *      q = [\boldsymbol a \cdot sin \frac \theta 2, cos \frac \theta 2]
+         *      q = [\boldsymbol a \cdot \sin(\frac{\theta}{2}), \cos(\frac{\theta}{2})]
          * @f]
          * @see @ref angle(), @ref axis(), @ref DualQuaternion::rotation(),
          *      @ref Matrix4::rotation(), @ref Complex::rotation(),
@@ -251,6 +318,15 @@ template<class T> class Quaternion {
             return Implementation::QuaternionConverter<T, U>::to(*this);
         }
 
+        /**
+         * @brief Raw data
+         * @return One-dimensional array of four elements
+         *
+         * @see @ref vector(), @ref scalar()
+         */
+        T* data() { return _vector.data(); }
+        constexpr const T* data() const { return _vector.data(); } /**< @overload */
+
         /** @brief Equality comparison */
         bool operator==(const Quaternion<T>& other) const {
             return _vector == other._vector && TypeTraits<T>::equals(_scalar, other._scalar);
@@ -273,17 +349,21 @@ template<class T> class Quaternion {
             return Implementation::isNormalizedSquared(dot());
         }
 
-        /** @brief Vector part */
-        constexpr const Vector3<T> vector() const { return _vector; }
+        /** @brief Vector part (@f$ \boldsymbol{q}_V @f$) */
+        Vector3<T>& vector() { return _vector; }
+        /* Returning const so it's possible to call constexpr functions on the
+           result. WTF, C++?! */
+        constexpr const Vector3<T> vector() const { return _vector; } /**< @overload */
 
-        /** @brief Scalar part */
-        constexpr T scalar() const { return _scalar; }
+        /** @brief Scalar part (@f$ q_S @f$) */
+        T& scalar() { return _scalar; }
+        constexpr T scalar() const { return _scalar; } /**< @overload */
 
         /**
          * @brief Rotation angle of unit quaternion
          *
          * Expects that the quaternion is normalized. @f[
-         *      \theta = 2 \cdot acos q_S
+         *      \theta = 2 \cdot \arccos(q_S)
          * @f]
          * @see @ref isNormalized(), @ref axis(), @ref rotation()
          */
@@ -515,6 +595,11 @@ template<class T> class Quaternion {
         Vector3<T> transformVectorNormalized(const Vector3<T>& vector) const;
 
     private:
+        #ifndef DOXYGEN_GENERATING_OUTPUT
+        /* Doxygen copies the description from Magnum::Quaternion here. Ugh. */
+        template<class> friend class Quaternion;
+        #endif
+
         /* Used to avoid including Functions.h */
         constexpr static T pow2(T value) {
             return value*value;
@@ -545,7 +630,7 @@ template<class T> inline Quaternion<T> operator/(T scalar, const Quaternion<T>&
     return {scalar/quaternion.vector(), scalar/quaternion.scalar()};
 }
 
-/** @debugoperator{Magnum::Math::Quaternion} */
+/** @debugoperator{Quaternion} */
 template<class T> Corrade::Utility::Debug& operator<<(Corrade::Utility::Debug& debug, const Quaternion<T>& value) {
     return debug << "Quaternion({" << Corrade::Utility::Debug::nospace
         << value.vector().x() << Corrade::Utility::Debug::nospace << ","
@@ -600,22 +685,25 @@ template<class T> Quaternion<T> quaternionFromMatrix(const Matrix3x3<T>& m) {
 
 template<class T> inline Quaternion<T> Quaternion<T>::rotation(const Rad<T> angle, const Vector3<T>& normalizedAxis) {
     CORRADE_ASSERT(normalizedAxis.isNormalized(),
-        "Math::Quaternion::rotation(): axis must be normalized", {});
+        "Math::Quaternion::rotation(): axis" << normalizedAxis << "is not normalized", {});
     return {normalizedAxis*std::sin(T(angle)/2), std::cos(T(angle)/2)};
 }
 
 template<class T> inline Quaternion<T> Quaternion<T>::fromMatrix(const Matrix3x3<T>& matrix) {
-    CORRADE_ASSERT(matrix.isOrthogonal(), "Math::Quaternion::fromMatrix(): the matrix is not orthogonal", {});
+    CORRADE_ASSERT(matrix.isOrthogonal(),
+        "Math::Quaternion::fromMatrix(): the matrix is not orthogonal:" << Corrade::Utility::Debug::newline << matrix, {});
     return Implementation::quaternionFromMatrix(matrix);
 }
 
 template<class T> inline Rad<T> Quaternion<T>::angle() const {
-    CORRADE_ASSERT(isNormalized(), "Math::Quaternion::angle(): quaternion must be normalized", {});
+    CORRADE_ASSERT(isNormalized(),
+        "Math::Quaternion::angle():" << *this << "is not normalized", {});
     return Rad<T>(T(2)*std::acos(_scalar));
 }
 
 template<class T> inline Vector3<T> Quaternion<T>::axis() const {
-    CORRADE_ASSERT(isNormalized(), "Math::Quaternion::axis(): quaternion must be normalized", {});
+    CORRADE_ASSERT(isNormalized(),
+        "Math::Quaternion::axis():" << *this << "is not normalized", {});
     return _vector/std::sqrt(1-pow2(_scalar));
 }
 
@@ -639,16 +727,56 @@ template<class T> inline Quaternion<T> Quaternion<T>::operator*(const Quaternion
 }
 
 template<class T> inline Quaternion<T> Quaternion<T>::invertedNormalized() const {
-    CORRADE_ASSERT(isNormalized(), "Math::Quaternion::invertedNormalized(): quaternion must be normalized", {});
+    CORRADE_ASSERT(isNormalized(),
+        "Math::Quaternion::invertedNormalized():" << *this << "is not normalized", {});
     return conjugated();
 }
 
 template<class T> inline Vector3<T> Quaternion<T>::transformVectorNormalized(const Vector3<T>& vector) const {
-    CORRADE_ASSERT(isNormalized(), "Math::Quaternion::transformVectorNormalized(): quaternion must be normalized", {});
+    CORRADE_ASSERT(isNormalized(),
+        "Math::Quaternion::transformVectorNormalized():" << *this << "is not normalized", {});
     const Vector3<T> t = T(2)*Math::cross(_vector, vector);
     return vector + _scalar*t + Math::cross(_vector, t);
 }
 
+namespace Implementation {
+
+template<class T> struct StrictWeakOrdering<Quaternion<T>> {
+    bool operator()(const Quaternion<T>& a, const Quaternion<T>& b) const {
+        StrictWeakOrdering<Vector3<T>> o;
+        if(o(a.vector(), b.vector()))
+            return true;
+        if(o(b.vector(), a.vector()))
+            return false;
+
+        return a.scalar() < b.scalar();
+    }
+};
+
+}
+
+}}
+
+namespace Corrade { namespace Utility {
+
+/** @configurationvalue{Magnum::Math::Quaternion} */
+template<class T> struct ConfigurationValue<Magnum::Math::Quaternion<T>> {
+    ConfigurationValue() = delete;
+
+    /** @brief Writes elements separated with spaces */
+    static std::string toString(const Magnum::Math::Quaternion<T>& value, ConfigurationValueFlags flags) {
+        return ConfigurationValue<Magnum::Math::Vector<4, T>>::toString(reinterpret_cast<const Magnum::Math::Vector<4, T>&>(value), flags);
+    }
+
+    /** @brief Reads elements separated with whitespace */
+    static Magnum::Math::Quaternion<T> fromString(const std::string& stringValue, ConfigurationValueFlags flags) {
+        const Magnum::Math::Vector<4, T> value = ConfigurationValue<Magnum::Math::Vector<4, T>>::fromString(stringValue, flags);
+        return reinterpret_cast<const Magnum::Math::Quaternion<T>&>(value);
+    }
+};
+
+/* No explicit instantiation needed, as Vector<4, T> is instantiated already */
+
 }}
 
 #endif
