@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -134,10 +134,7 @@ template<UnsignedInt dimensions, class T> class Range {
          * Performs only default casting on the values, no rounding or
          * anything else. Example usage:
          *
-         * @code{.cpp}
-         * Range2D<Float> floatingPoint({1.3f, 2.7f}, {-15.0f, 7.0f});
-         * Range2D<Byte> integral(floatingPoint); // {{1, 2}, {-15, 7}}
-         * @endcode
+         * @snippet MagnumMath.cpp Range-conversion
          */
         template<class U> constexpr explicit Range(const Range<dimensions, U>& other) noexcept: _min(other._min), _max(other._max) {}
 
@@ -759,6 +756,21 @@ extern template MAGNUM_EXPORT Corrade::Utility::Debug& operator<<(Corrade::Utili
 extern template MAGNUM_EXPORT Corrade::Utility::Debug& operator<<(Corrade::Utility::Debug&, const Range<3, Double>&);
 #endif
 
+namespace Implementation {
+
+template<UnsignedInt dimensions, class T> struct StrictWeakOrdering<Range<dimensions, T>> {
+    bool operator()(const Range<dimensions, T>& a, const Range<dimensions, T>& b) const {
+        StrictWeakOrdering<typename Range<dimensions, T>::VectorType> o;
+        if(o(a.min(), b.min()))
+            return true;
+        if(o(b.min(), a.min()))
+            return false;
+        return o(a.max(), b.max());
+    }
+};
+
+}
+
 }}
 
 namespace Corrade { namespace Utility {
