@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -45,7 +45,7 @@ namespace Magnum { namespace Primitives {
 Sphere with radius @cpp 1.0f @ce. Indexed @ref MeshPrimitive::Triangles with
 normals.
 
-@image html primitives-icospheresolid.png
+@image html primitives-icospheresolid.png width=256px
 
 The @p subdivisions parameter describes how many times is each icosphere
 triangle subdivided, recursively. Specifying @cpp 0 @ce will result in an
