@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -37,10 +37,6 @@
 #include "Magnum/SceneGraph/SceneGraph.h"
 #include "Magnum/SceneGraph/visibility.h"
 
-#ifdef MAGNUM_BUILD_DEPRECATED
-#include <Corrade/Utility/Macros.h>
-#endif
-
 namespace Magnum { namespace SceneGraph {
 
 /**
@@ -53,22 +49,13 @@ subclass instead. See also @ref scenegraph for more information.
 Uses @ref Corrade::Containers::LinkedList for efficient feature management.
 Traversing through the feature list can be done using range-based for:
 
-@code{.cpp}
-AbstractObject3D object;
-for(AbstractFeature3D& feature: object.features()) {
-    // ...
-}
-@endcode
+@snippet MagnumSceneGraph.cpp AbstractObject-features-range
 
 Or, if you need more flexibility, like in the following code. It is also
 possible to go in reverse order using @ref Corrade::Containers::LinkedList::last()
 and @ref AbstractFeature::previousFeature().
 
-@code{.cpp}
-for(AbstractFeature3D* feature = object.features().first(); feature; feature = feature->nextFeature()) {
-    // ...
-}
-@endcode
+@snippet MagnumSceneGraph.cpp AbstractObject-features
 
 @section SceneGraph-AbstractObject-explicit-specializations Explicit template specializations
 
@@ -118,43 +105,13 @@ template<UnsignedInt dimensions, class T> class AbstractObject
             return static_cast<const Containers::LinkedList<AbstractFeature<dimensions, T>>&>(*this);
         }
 
-        #ifdef MAGNUM_BUILD_DEPRECATED
-        /**
-         * @brief Whether this object has features
-         * @deprecated Use `features().isEmpty()` instead.
-         */
-        CORRADE_DEPRECATED("use features().isEmpty() instead") bool hasFeatures() const { return !features().isEmpty(); }
-
-        /**
-         * @brief First object feature or `nullptr`, if this object has no features
-         * @deprecated Use `features().first()` instead.
-         */
-        CORRADE_DEPRECATED("use features().first() instead") FeatureType* firstFeature() { return features().first(); }
-
-        /** @overload
-         * @deprecated Use `features().first()` instead.
-         */
-        CORRADE_DEPRECATED("use features().first() instead") const FeatureType* firstFeature() const { return features().first(); }
-
-        /**
-         * @brief Last object feature or `nullptr`, if this object has no features
-         * @deprecated Use `features().last()` instead.`
-         */
-        CORRADE_DEPRECATED("use features().last() instead") FeatureType* lastFeature() { return features().last(); }
-
-        /** @overload
-         * @deprecated Use `features().last()` instead.
-         */
-        CORRADE_DEPRECATED("use features().last() instead") const FeatureType* lastFeature() const { return features().last(); }
-        #endif
-
         /**
          * @brief Add a feature
          *
          * Calling `object.addFeature<MyFeature>(args...)` is equivalent to
          * `new MyFeature{object, args...}`.
          */
-        template<class U, class ...Args> U& addFeature(Args... args) {
+        template<class U, class ...Args> U& addFeature(Args&&... args) {
             return *(new U{*this, std::forward<Args>(args)...});
         }
 
