@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -26,11 +26,9 @@
 */
 
 /** @file
- * @brief Class @ref Magnum::SceneGraph::AbstractTransformation, alias @ref Magnum::SceneGraph::AbstractBasicTransformation2D, @ref Magnum::SceneGraph::AbstractBasicTransformation3D, typedef @ref Magnum::SceneGraph::AbstractTransformation2D, @ref Magnum::SceneGraph::AbstractTransformation3D, enum @ref Magnum::SceneGraph::TransformationType
+ * @brief Class @ref Magnum::SceneGraph::AbstractTransformation, alias @ref Magnum::SceneGraph::AbstractBasicTransformation2D, @ref Magnum::SceneGraph::AbstractBasicTransformation3D, typedef @ref Magnum::SceneGraph::AbstractTransformation2D, @ref Magnum::SceneGraph::AbstractTransformation3D
  */
 
-#include <Corrade/Utility/Macros.h>
-
 #include "Magnum/SceneGraph/SceneGraph.h"
 #include "Magnum/SceneGraph/visibility.h"
 
@@ -88,21 +86,6 @@ template<UnsignedInt dimensions, class T> class AbstractTransformation {
         virtual void doResetTransformation() = 0;
 };
 
-#ifdef MAGNUM_BUILD_DEPRECATED
-/**
-@brief Transformation type
-@deprecated Use `*Transformation*::*()` and `*Transformation::*Local*()`
-    overloads instead.
-*/
-enum class CORRADE_DEPRECATED_ENUM("use *() and *Local() overloads instead") TransformationType: UnsignedByte {
-    /** Global transformation, applied after all other transformations. */
-    Global = 0x00,
-
-    /** Local transformation, applied before all other transformations. */
-    Local = 0x01
-};
-#endif
-
 /**
 @brief Base transformation for two-dimensional scenes
 
