@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -33,7 +33,7 @@
 #include <unordered_map>
 
 #include "Magnum/Math/Range.h"
-#include "Magnum/Texture.h"
+#include "Magnum/GL/Texture.h"
 #include "Magnum/Text/visibility.h"
 
 namespace Magnum { namespace Text {
@@ -48,13 +48,7 @@ Contains font glyphs prerendered into texture atlas.
 Create GlyphCache object with sufficient size and then call
 @ref AbstractFont::createGlyphCache() to fill it with glyphs.
 
-@code{.cpp}
-Text::AbstractFont* font;
-Text::GlyphCache* cache = new GlyphCache(Vector2i(512));
-font->createGlyphCache(cache, "abcdefghijklmnopqrstuvwxyz"
-                              "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
-                              "0123456789?!:;,. ");
-@endcode
+@snippet MagnumText.cpp GlyphCache-usage
 
 See @ref Renderer for information about text rendering.
 @todo Some way for Font to negotiate or check internal texture format
@@ -74,31 +68,35 @@ class MAGNUM_TEXT_EXPORT GlyphCache {
          * although the actual glyph cache texture has @p size. Glyph
          * @p padding can be used to account for e.g. glyph shadows.
          */
-        explicit GlyphCache(TextureFormat internalFormat, const Vector2i& originalSize, const Vector2i& size, const Vector2i& padding = Vector2i());
+        explicit GlyphCache(GL::TextureFormat internalFormat, const Vector2i& originalSize, const Vector2i& size, const Vector2i& padding);
 
         /**
          * @brief Constructor
          *
          * Same as calling the above with @p originalSize and @p size the same.
          */
-        explicit GlyphCache(TextureFormat internalFormat, const Vector2i& size, const Vector2i& padding = Vector2i());
+        explicit GlyphCache(GL::TextureFormat internalFormat, const Vector2i& size, const Vector2i& padding = {});
 
         /**
          * @brief Constructor
          *
          * Sets internal texture format to red channel only. On desktop OpenGL
-         * requires @extension{ARB,texture_rg} (also part of OpenGL ES 3.0), in
-         * ES2 uses @extension{EXT,texture_rg}, if available, or
-         * @ref TextureFormat::Luminance as fallback.
+         * requires @gl_extension{ARB,texture_rg} (also part of OpenGL ES 3.0 and
+         * WebGL 2), on ES2 unconditionally uses @ref GL::TextureFormat::Luminance.
+         * This is done for consistency with @ref GL::pixelFormat(), which
+         * unconditionally returns @ref GL::PixelFormat::Luminance for
+         * @ref PixelFormat::R8Unorm. See
+         * @ref GlyphCache(GL::TextureFormat, const Vector2i&, const Vector2i&)
+         * for an alternative.
          */
-        explicit GlyphCache(const Vector2i& originalSize, const Vector2i& size, const Vector2i& padding = Vector2i());
+        explicit GlyphCache(const Vector2i& originalSize, const Vector2i& size, const Vector2i& padding);
 
         /**
          * @brief Constructor
          *
          * Same as calling the above with @p originalSize and @p size the same.
          */
-        explicit GlyphCache(const Vector2i& size, const Vector2i& padding = Vector2i());
+        explicit GlyphCache(const Vector2i& size, const Vector2i& padding = {});
 
         virtual ~GlyphCache();
 
@@ -116,7 +114,7 @@ class MAGNUM_TEXT_EXPORT GlyphCache {
         std::size_t glyphCount() const { return glyphs.size(); }
 
         /** @brief Cache texture */
-        Texture2D& texture() { return _texture; }
+        GL::Texture2D& texture() { return _texture; }
 
         /**
          * @brief Parameters of given glyph
@@ -189,10 +187,10 @@ class MAGNUM_TEXT_EXPORT GlyphCache {
         virtual void setImage(const Vector2i& offset, const ImageView2D& image);
 
     private:
-        void MAGNUM_LOCAL initialize(TextureFormat internalFormat, const Vector2i& size);
+        void MAGNUM_LOCAL initialize(GL::TextureFormat internalFormat, const Vector2i& size);
 
         Vector2i _size, _padding;
-        Texture2D _texture;
+        GL::Texture2D _texture;
 
         std::unordered_map<UnsignedInt, std::pair<Vector2i, Range2Di>> glyphs;
 };
