@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -35,8 +35,20 @@ namespace Magnum { namespace Trade {
 
 #ifndef DOXYGEN_GENERATING_OUTPUT
 class AbstractImageConverter;
+
+enum class ImporterFileCallbackPolicy: UnsignedByte;
 class AbstractImporter;
+
+enum class MaterialType: UnsignedByte;
+enum class MaterialAlphaMode: UnsignedByte;
 class AbstractMaterialData;
+
+enum class AnimationTrackTargetType: UnsignedByte;
+enum class AnimationTrackType: UnsignedByte;
+class AnimationTrackData;
+class AnimationData;
+
+enum class CameraType: UnsignedByte;
 class CameraData;
 
 template<UnsignedInt> class ImageData;
